/* -------------------------------------------------------------------
 *
 *	File: mSQL++.cpp
 *
 *	Date: 99/05/28
 *
 *	Author: Jrgen Schober, Point Design Software
 *
 *	Description: C++ wrapper for the msql.library.
 *			At least msql.library V5.0 is requiered.
 *			#undef _USE_ADAC_EXCEPTION_ to switch off
 *			Error-handling.
 *
 * ------------------------------------------------------------------- */

#include <stdio.h>
#include <string.h>

#include <libraries/mSQL++.h>

struct Library *MsqlBase = NULL;


mSQL::mSQL()
{
	sql_Connect = NULL;
	sql_IsConnected = FALSE;

	MsqlBase = OpenLibrary("msql.library", 5);
	if (MsqlBase)
	{
		sql_Connect = MsqlAllocConnection();

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::mSQL()");
#endif
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_FAILED_ON_INIT,
				"mSQL::mSQL() failed to open msql.library (V5.0+ required!)"));
#endif
}

mSQL::~mSQL()
{
	if (MsqlBase)
	{
		CloseLibrary(MsqlBase);
		MsqlBase = NULL;
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_FAILED_ON_INIT,
				"mSQL::~mSQL() called with invalid MsqlBase pointer."));
#endif
}

BOOL mSQL::AddMHook(Hook* hook, TagList *tl)
{
	if (MsqlBase)
		return MsqlAddMHookA(hook,tl);
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_FAILED_ON_INIT,
				"mSQL::AddMHookA() called with invalid MsqlBase pointer."));
#endif

	return FALSE;
}

BOOL mSQL::AddMHook(Hook* hook, ULONG varargs, ...)
{
	return AddMHook(hook,(TagList*)&varargs);
}

void mSQL::RemMHook(Hook* hook)
{
	if (MsqlBase) MsqlRemMHook(hook);
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_FAILED_ON_INIT,
				"mSQL::RemMHookA() called with invalid MsqlBase pointer."));
#endif

}

void mSQL::AllocConnection()
{
	if (MsqlBase)
	{
		sql_Connect = MsqlAllocConnection();

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::AllocConnection()");
#endif
	}

#ifdef _USE_ADAC_EXCEPTION_
	if (!sql_Connect)
		throw(AdacException(ADACE_FAILED_ON_INIT,
			"mSQL::AllocConnection() failed to allocate connection handle"));
#endif
}

void mSQL::FreeConnection()
{
	if (sql_Connect) 
	{
		Close();
		MsqlFreeConnection(sql_Connect);

		sql_Connect = NULL;
	}
}

void mSQL::Close()
{
	if (sql_Connect) 
	{
		if (sql_IsConnected)
		{
			MsqlClose(sql_Connect);
			sql_IsConnected = FALSE;
		}
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::Close() called with invalid connection handle!"));
#endif
}

void mSQL::Connect(char* host)
{
	if (sql_Connect)
	{
		Close();

		sql_Connect = MsqlConnect(sql_Connect,host);
		if (sql_Connect) sql_IsConnected = TRUE;
			
#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::Connect()");
#endif

	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::Connect() called without connection handle!"));
#endif
}

int mSQL::CopyDB(char* dbFrom, char* dbTo)
{
	int r = 0;

	if (sql_Connect)
	{
		r = MsqlCopyDB(sql_Connect,dbFrom,dbTo);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::CopyDB()");
#endif

	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::CopyDB() called without connection handle!"));
#endif
	
	return r;
}

int mSQL::CreateDB(char* dbName)
{
	int r = 0;

	if (sql_Connect)
	{
		r = MsqlCreateDB(sql_Connect,dbName);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::CreateDB()");
#endif
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::CreateDB() called without connection handle!"));
#endif
	
	return 0;
}

int mSQL::DropDB(char* dbName)
{
	int r = 0;

	if (sql_Connect)
	{
		r = MsqlDropDB(sql_Connect,dbName);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::DropDB()");
#endif
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::DropDB() called without connection handle!"));
#endif	
	return r;
}

int mSQL::MoveDB(char* dbFrom, char* dbTo)
{
	int r = 0;

	if (sql_Connect)
	{
		r = MsqlMoveDB(sql_Connect,dbFrom,dbTo);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::MoveDB()");
#endif
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::CopyDB() called without connection handle!"));
#endif
	
	return r;
}

int mSQL::SelectDB(char* dbName)
{
	int r = 0;

	if (sql_Connect)
	{
		r = MsqlSelectDB(sql_Connect,dbName);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::SelectDB()");
#endif

	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::SelectDB() called without connection handle!"));
#endif
	
	return r;
}

void mSQL::DataSeek(m_result* res,int pos)
{
	if (sql_Connect)
	{
		if (res != NULL)
		{
			MsqlDataSeek(res,pos);

#ifdef _USE_ADAC_EXCEPTION_
			char* sql_ErrorMsg = GetErrMsg();
			if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
				throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::DataSeek()");
#endif

		}
#ifdef _USE_ADAC_EXCEPTION_
		else
			throw(AdacException(ADACE_INTERNAL_FAILURE,
				"mSQL::DataSeek() got invalid result handle!"));
#endif
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::DataSeek() got invalid connection handle!"));
#endif
}

void mSQL::FieldSeek(m_result* res,int pos)
{
	if (sql_Connect)
	{
		if (res != NULL)
		{
			MsqlFieldSeek(res,pos);

#ifdef _USE_ADAC_EXCEPTION_
			char* sql_ErrorMsg = GetErrMsg();
			if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
				throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::FieldSeek()");
#endif

		}
#ifdef _USE_ADAC_EXCEPTION_
		else
			throw(AdacException(ADACE_INTERNAL_FAILURE,
				"mSQL::FieldSeek() got invalid result handle!"));
#endif
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::FieldSeek() got invalid connection handle!"));
#endif
}


m_field* mSQL::FetchField(m_result* res)
{
	m_field *f = NULL;

	if (sql_Connect)
	{
		if (res != NULL)
		{
			f = MsqlFetchField(res);

#ifdef _USE_ADAC_EXCEPTION_
			char* sql_ErrorMsg = GetErrMsg();
			if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
				throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::FetchField()");
#endif

		}
#ifdef _USE_ADAC_EXCEPTION_
		else
			throw(AdacException(ADACE_INTERNAL_FAILURE,
				"mSQL::FetchField() got invalid result handle!"));
#endif
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::FetchField() got invalid connection handle!"));
#endif

	return f;
}

m_row mSQL::FetchRow(m_result *res)
{
	m_row r = NULL;

	if (sql_Connect)
	{
		if (res != NULL)
		{
			r = MsqlFetchRow(res);

#ifdef _USE_ADAC_EXCEPTION_
			char* sql_ErrorMsg = GetErrMsg();
			if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
				throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::FetchRow()");
#endif

		}
#ifdef _USE_ADAC_EXCEPTION_
		else
			throw(AdacException(ADACE_INTERNAL_FAILURE,
				"mSQL::FetchRow() got invalid result handle!"));
#endif
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::FetchRow() got invalid connection handle!"));
#endif

	return r;
}

m_result* mSQL::ListDBs()
{
	m_result *r = NULL;

	if (sql_Connect)
	{
		r = MsqlListDBs(sql_Connect);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::ListDB()");
#endif
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::ListDBs() got invalid connection handle!"));
#endif

	return r;
}

m_result* mSQL::ListTables()
{
	m_result *r = NULL;

	if (sql_Connect)
	{
		r = MsqlListTables(sql_Connect);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::ListTables()");
#endif

	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::LisTables() got invalid connection handle!"));
#endif

	return r;
}

m_result* mSQL::ListFields(char *table)
{
	m_result *r = NULL;

	if (sql_Connect)
	{
		r = MsqlListFields(sql_Connect,table);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::ListFields()");
#endif
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::ListFields() got invalid connection handle!"));
#endif
	return r;
}

m_result* mSQL::ListIndex(char* table,char* idx)
{
	m_result *r = NULL;

	if (sql_Connect)
	{
		r = MsqlListIndex(sql_Connect,table,idx);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::ListIndex()");
#endif
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::ListIndex() got invalid connection handle!"));
#endif

	return r;
}

int mSQL::NumFields(m_result* res)
{
	int r = 0;

	if (res)
	{
		r = MsqlNumFields(res);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::NumFields()");
#endif

	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::NumField() got invalid result handle!"));
#endif

	return r;
}

int mSQL::NumRows(m_result* res)
{
	int r = 0;

	if (res)
	{
		r = MsqlNumRows(res);

/*
	This is a makro, so a Connection might be invalid, but the
	function doesn't return an Error -> here the Exception handling
	returns an Error in this case ! (you might switch this off)
*/

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::NumRows()");
#endif
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::NumRows() got invalid result handle!"));
#endif

	return r;
}

int mSQL::Query(char* query)
{
	int r = 0;

	if (sql_Connect)
	{
		r = MsqlQuery(sql_Connect,query);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::Query()");
#endif

	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::Query() got invalid connection handle!"));
#endif

	return r;
}

m_result* mSQL::StoreResult()
{
	m_result *r = NULL;

	if (sql_Connect)
	{
		r = MsqlStoreResult(sql_Connect);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::StoreResult()");
#endif

	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::StoreResult() got invalid connection handle!"));
#endif

	return r;
}

void mSQL::FreeResult(m_result* res)
{
	if (res)
	{
		MsqlFreeResult(res);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::FreeResult()");
#endif
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::FreeResult() got invalid result handle!"));
#endif
}

char* mSQL::GetErrMsg()
{
	char *sql_ErrorMsg = NULL;

	if (sql_Connect)
	{
		sql_ErrorMsg = MsqlGetErrMsg(sql_Connect);

#ifdef _USE_ADAC_EXCEPTION_
// This doesn't make sence. If you want the error message,
// you don't want to get an exception,
// though with exception enabled, you will hardly get the 
// message at all...

//		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
//			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg);
#endif
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::GetErrMsg() got invalid connection handle!"));
#endif

	return sql_ErrorMsg;
}

char* mSQL::DateOffset(char* start, int day, int month, int year)
{
	char *c = NULL;

	if (MsqlBase)
	{
		c = MsqlDateOffset(start,day,month,year);
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::DateOffset() called with invalid MsqlBase pointer!"));
#endif

	return c;
}

time_t mSQL::DateToUnixTime(char* date)
{
	time_t t = 0;

	if (MsqlBase)
	{
		t = MsqlDateToUnixTime(date);
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::DateToUnix() called with invalid MsqlBase pointer!"));
#endif

	return t;
}

int mSQL::DiffDates(char* date1, char* date2)
{
	int i = 0;

	if (MsqlBase)
	{
		i = MsqlDiffDates(date1,date2);
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::DiffDates() called with invalid MsqlBase pointer!"));
#endif

	return i;
}

char* mSQL::DiffTimes(char* time1, char* time2)
{
	char *c = NULL;

	if (MsqlBase)
	{
		c = MsqlDiffTimes(time1,time2);
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::DiffTimes() called with invalid MsqlBase pointer!"));
#endif

	return c;
}

char* mSQL::SumTimes(char* time1, char* time2)
{
	char *c;

	if (MsqlBase)
	{
		c = MsqlSumTimes(time1,time2);
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::SumTimes() called with invalid MsqlBase pointer!"));
#endif

	return c;
}

time_t mSQL::TimeToUnixTime(char* date)
{
	time_t t = 0;

	if (MsqlBase)
	{
		t = MsqlTimeToUnixTime(date);
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::TimeToUnixTime() called with invalid MsqlBase pointer!"));
#endif

	return t;
}

char* mSQL::UnixTimeToDate(time_t time)
{
	char *c = NULL;

	if (MsqlBase)
	{
		c = MsqlUnixTimeToDate(time);
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::UnixTimeToDate() called with invalid MsqlBase pointer!"));
#endif

	return c;
}

char* mSQL::UnixTimeToTime(time_t time)
{
	char *c = NULL;

	if (MsqlBase)
	{
		c = MsqlUnixTimeToTime(time);
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::UnixTimeToTime() called with invalid MsqlBase pointer!"));
#endif

	return c;
}

char* mSQL::GetCharConf(char *y,char *z)
{
	char *c = NULL;

	if (sql_Connect)
	{
		c = MsqlGetCharConf(sql_Connect,y,z);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::GetCharConf()");
#endif

	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::GetCharConf() got invalid connection handle!"));
#endif

	return c;
}

int mSQL::GetIntConf(char *y,char *z)
{
	int i = 0;

	if (sql_Connect)
	{
		i = MsqlGetIntConf(sql_Connect,y,z);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::GetIntConf()");
#endif

	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::GetIntConf() got invalid connection handle!"));
#endif

	return i;
}

int mSQL::GetProtoInfo()
{
	int i = 0;

	if (sql_Connect)
	{
		i = MsqlGetProtoInfo(sql_Connect);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::GetProtoInfo()");
#endif

	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::GetProtoInfo() got invalid connection handle!"));
#endif

	return i;
}

char* mSQL::GetHostInfo()
{
	char *c = NULL;

	if (sql_Connect)
	{
		c = MsqlGetHostInfo(sql_Connect);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::GetHostInfo()");
#endif

	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::GetHostInfo() got invalid connection handle!"));
#endif

	return c;
}

m_seq* mSQL::GetSequenceInfo(char* table)
{
	m_seq *s = NULL;

	if (sql_Connect)
	{
		s = MsqlGetSequenceInfo(sql_Connect,table);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::GetSequenceInfo()");
#endif

	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::GetSequenceInfo() got invalid connection handle!"));
#endif

	return s;
}

char* mSQL::GetServerInfo()
{
	char *c = NULL;

	if (sql_Connect)
	{
		c = MsqlGetServerInfo(sql_Connect);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::GetServerInfo()");
#endif

	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::GetServerInfo() got invalid connection handle!"));
#endif

	return c;
}

int mSQL::GetServerStats(char* buffer, ULONG size)
{
	int i = 0;

	if (sql_Connect)
	{
		i = MsqlGetServerStats(sql_Connect,buffer,size);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::GetServerStats()");
#endif
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::GetServerStats() got invalid connection handle!"));
#endif

	return i;
}

int mSQL::LoadConfigFile(char *fn)
{
	int i = 0;

	if (sql_Connect)
	{
		i = MsqlLoadConfigFile(sql_Connect,fn);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::LoadConfigFile()");
#endif

	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::LoadConfigFile() got invalid connection handle!"));
#endif

	return i;
}

int mSQL::ReloadAcls()
{
	int i = 0;

	if (sql_Connect)
	{
		i = MsqlReloadAcls(sql_Connect);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::ReloadAcls()");
#endif
	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"mSQL::ReloadAcls() got invalid connection handle!"));
#endif
	return i;
}

int mSQL::Shutdown()
{
	int i = 0;
	if (sql_Connect)
	{
		i = MsqlShutdown(sql_Connect);

#ifdef _USE_ADAC_EXCEPTION_
		char* sql_ErrorMsg = GetErrMsg();
		if (sql_ErrorMsg && (strcmp(sql_ErrorMsg,"") != 0))
			throw AdacException(ADACE_SQLERROR,sql_ErrorMsg,"mSQL::Shutdown()");
#endif

	}
#ifdef _USE_ADAC_EXCEPTION_
	else
		throw(AdacException(ADACE_INTERNAL_FAILURE,
			"ShutDown() got invalid connection handle!"));
#endif

	return i;
}

char* mSQL::GetField(m_row row,int pos)
{
	// Warning! This Function is dangerous!!!
	// Please be sure how many fields you have in rows[]!
	if (row)
	{
		return row[pos];
	}
	else
		return NULL;
}

int mSQL::GetFieldInfo(m_field* fld)
{
	if (fld)
	{
		return fld->type;
	}
	return 0;
}

BOOL mSQL::IsNotNull(m_field* fld)
{
	if (fld)
	{
		return IS_NOT_NULL(fld->flags);
	}
	return TRUE;
}
	
BOOL mSQL::IsUnique(m_field* fld)
{
	if (fld)
	{
		return IS_UNIQUE(fld->flags);
	}
	return TRUE;
}

